from django.core.exceptions import ValidationError
from django.test import TestCase

from app.models import Boolean
from django.db import DatabaseError


class BooleanTests(TestCase):

    @classmethod
    def setUpClass(cls):
        Boolean.objects.create(boolean=True)
        Boolean.objects.create(boolean=False)

    def test_boolean_create(self):
        true = Boolean.objects.get(boolean=True)
        false = Boolean.objects.get(boolean=False)
        self.assertTrue(true.boolean)
        self.assertFalse(false.boolean)

    def test_boolean_exact_lookup(self):
        self.assertTrue(Boolean.objects.filter(boolean__exact=True).exists())

    def test_boolean_in_lookup(self):
        self.assertEqual(Boolean.objects.filter(boolean__in=[True, False]).count(), 2)

    def test_boolean_isnull_lookup(self):
        self.assertEqual(Boolean.objects.filter(boolean__isnull=False).count(), 2)

    def test_boolean_incorrect_values(self):
        with self.assertRaises(ValidationError):
            Boolean.objects.create(boolean='string value')
        with self.assertRaises(DatabaseError):  # Why DatabaseError?
            Boolean.objects.create(boolean=None)
        with self.assertRaises(ValidationError):
            Boolean.objects.create(boolean=100)


import time


def wait_for_tarantool(connection):
    while True:
        try:
            cursor = connection.cursor()
            cursor.execute("SELECT 1")
        except Exception as e:
            time.sleep(.5)
            continue
        finally:
            break



# Generated by Django 3.0.7 on 2020-06-27 09:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='biginteger',
            name='integer',
        ),
    ]



import datetime

from django.core.management import BaseCommand
from model_bakery import baker

from ...models import *

ITERATIONS = 10000


class Command(BaseCommand):

    def handle(self, *args, **options):
        t = datetime.datetime.now()
        # models_list = [ForeignKey, Text, Integer, Char, Float, GenericIPAddress, Uuid, Slug, DateTime, Date, Time]
        # for model in models_list:
        # baker.make(AllModel, ITERATIONS)
        for _ in range(30000):
            Integer.objects.create(integer=123123)
        dt = datetime.datetime.now() - t

        print(f"Insertion took {dt}")


from django.db import DatabaseError
from django.test import TestCase

from app.models import Integer


class IntegerTests(TestCase):
    def setUp(self):
        self.int0_id = Integer.objects.create(integer=0).id
        self.int1_id = Integer.objects.create(integer=1111).id

    def test_create_integer(self):
        int0 = Integer.objects.get(id=self.int0_id)
        int1 = Integer.objects.get(id=self.int1_id)
        self.assertEqual(int0.integer, 0)
        self.assertEqual(int1.integer, 1111)
        self.assertLess(int0.integer, int1.integer)
        self.assertGreater(int1.integer, int0.integer)

    def test_extremal_values(self):
        int_biggest = Integer.objects.create(integer=18446744073709551615)
        self.assertEqual(int_biggest.integer, 18446744073709551615)
        int_smallest = Integer.objects.create(integer=-9223372036854775808)
        self.assertEqual(int_smallest.integer, -9223372036854775808)
        self.assertLess(int_smallest.integer, int_biggest.integer)
        with self.assertRaises(Exception):
            Integer.objects.create(integer=18446744073709551616)
        with self.assertRaises(Exception):
            Integer.objects.create(integer=-9223372036854776840)



import decimal

from django.test import TestCase

from app.models import Decimal


class DecimalTests(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.decimal_value = 55.33
        cls.decimal_item_id = Decimal.objects.create(decimal=cls.decimal_value).id

    def test_char_create(self):
        decimal_item = Decimal.objects.get(id=self.decimal_item_id)
        self.assertEqual(decimal_item.decimal, self.decimal_value)

    def test_invalid_decimal(self):
        decimal_invalid_value = 55555.11111
        with self.assertRaises(decimal.InvalidOperation):
            Decimal.objects.create(decimal=decimal_invalid_value)


import subprocess

from django.db.backends.base.client import BaseDatabaseClient


class DatabaseClient(BaseDatabaseClient):
    executable_name = 'tarantoolctl'

    @classmethod
    def settings_to_cmd_args(cls, settings_dict):
        args = [cls.executable_name]

        user = settings_dict['OPTIONS'].get('user', settings_dict['USER'])
        passwd = settings_dict['OPTIONS'].get('passwd', settings_dict['PASSWORD'])
        host = settings_dict['OPTIONS'].get('host', settings_dict['HOST'])
        port = settings_dict['OPTIONS'].get('port', settings_dict['PORT'])

        args += ["connect", "tcp://%s:%s@%s:%s" % (user, passwd, host, port)]
        return args

    def runshell(self):
        args = DatabaseClient.settings_to_cmd_args(self.connection.settings_dict)
        subprocess.run(args, check=True)




import uuid

from django.core.exceptions import ValidationError
from django.test import TestCase

from app.models import Uuid


class UuidTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.uuid_value = uuid.uuid4()
        Uuid.objects.create(uuid=cls.uuid_value)

    def test_uuid(self):
        uuid_item = Uuid.objects.get(uuid=self.uuid_value)
        self.assertEqual(uuid_item.uuid, self.uuid_value)

    def test_uuid_exact(self):
        uuid_item = Uuid.objects.get(uuid__exact=self.uuid_value)
        self.assertEqual(uuid_item.uuid, self.uuid_value)

    def test_uuid_in(self):
        uuid_item = Uuid.objects.get(uuid__in=[self.uuid_value])
        self.assertEqual(uuid_item.uuid, self.uuid_value)

    def test_invalid_uuid(self):
        with self.assertRaises(ValidationError):
            Uuid.objects.create(uuid='non-uuid text value')



import copy
from random import choice
from string import ascii_lowercase

from django.apps.registry import Apps
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.backends.ddl_references import Statement
from django.db.backends.utils import strip_quotes
from django.db.models import UniqueConstraint


class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):

    sql_delete_table = "DROP TABLE %(table)s"
    sql_create_fk = None
    sql_create_inline_fk = "REFERENCES %(to_table)s (%(to_column)s) DEFERRABLE INITIALLY DEFERRED"
    sql_create_unique = "CREATE UNIQUE INDEX %(name)s ON %(table)s (%(columns)s)"
    sql_delete_unique = 'DROP INDEX %(name)s ON %(table)s'

    def quote_value(self, value):
        if value is None:
            return "NULL"
        if isinstance(value, bool):
            return str(value)
        return "'%s'" % value

    def _is_referenced_by_fk_constraint(self, table_name, column_name=None, ignore_self=False):
        """
        Return whether or not the provided table name is referenced by another
        one. If `column_name` is specified, only references pointing to that
        column are considered. If `ignore_self` is True, self-referential
        constraints are ignored.
        """
        with self.connection.cursor() as cursor:
            for other_table in self.connection.introspection.get_table_list(cursor):
                if ignore_self and other_table.name == table_name:
                    continue
                constraints = self.connection.introspection._get_foreign_key_constraints(cursor, other_table.name)
                for constraint in constraints.values():
                    constraint_table, constraint_column = constraint['foreign_key']
                    if (constraint_table == table_name and
                            (column_name is None or constraint_column == column_name)):
                        return True
        return False

    def _remake_table(self, model, create_field=None, delete_field=None, alter_field=None):
        """
        Shortcut to transform a model from old_model into new_model

        The essential steps are:
          1. Create a table with the updated definition called "new__(rand_name)__app_model"
          2. Copy the data from the existing "app_model" table to the new table
          3. If there is foreign key that references the table, drop the referencing constraints
          4. Drop the "app_model" table
          5. Rename the "new__(rand_name)__app_model" table to "app_model"
          6. Restore any index of the previous "app_model" table.
        """
        # Self-referential fields must be recreated rather than copied from
        # the old model to ensure their remote_field.field_name doesn't refer
        # to an altered field.
        def is_self_referential(f):
            return f.is_relation and f.remote_field.model is model
        # Work out the new fields dict / mapping
        body = {
            f.name: f.clone() if is_self_referential(f) else f
            for f in model._meta.local_concrete_fields
        }
        # Since mapping might mix column names and default values,
        # its values must be already quoted.
        mapping = {f.column: self.quote_name(f.column) for f in model._meta.local_concrete_fields}
        # This maps field names (not columns) for things like unique_together
        rename_mapping = {}
        # If any of the new or altered fields is introducing a new PK,
        # remove the old one
        restore_pk_field = None
        if getattr(create_field, 'primary_key', False) or (
                alter_field and getattr(alter_field[1], 'primary_key', False)):
            for name, field in list(body.items()):
                if field.primary_key:
                    field.primary_key = False
                    restore_pk_field = field
                    if field.auto_created:
                        del body[name]
                        del mapping[field.column]
        # Add in any created fields
        if create_field:
            body[create_field.name] = create_field
            # Choose a default and insert it into the copy map
            if not create_field.many_to_many and create_field.concrete:
                mapping[create_field.column] = self.quote_value(
                    self.effective_default(create_field)
                )
        # Add in any altered fields
        if alter_field:
            old_field, new_field = alter_field
            body.pop(old_field.name, None)
            mapping.pop(old_field.column, None)
            body[new_field.name] = new_field
            if old_field.null and not new_field.null:
                case_sql = "coalesce(%(col)s, %(default)s)" % {
                    'col': self.quote_name(old_field.column),
                    'default': self.quote_value(self.effective_default(new_field))
                }
                mapping[new_field.column] = case_sql
            else:
                mapping[new_field.column] = self.quote_name(old_field.column)
            rename_mapping[old_field.name] = new_field.name
        # Remove any deleted fields
        if delete_field:
            del body[delete_field.name]
            del mapping[delete_field.column]
            # Remove any implicit M2M tables
            if delete_field.many_to_many and delete_field.remote_field.through._meta.auto_created:
                return self.delete_model(delete_field.remote_field.through)
        # Work inside a new app registry
        apps = Apps()

        # Work out the new value of unique_together, taking renames into
        # account
        unique_together = [
            [rename_mapping.get(n, n) for n in unique]
            for unique in model._meta.unique_together
        ]

        # Work out the new value for index_together, taking renames into
        # account
        index_together = [
            [rename_mapping.get(n, n) for n in index]
            for index in model._meta.index_together
        ]

        indexes = model._meta.indexes
        if delete_field:
            indexes = [
                index for index in indexes
                if delete_field.name not in index.fields
            ]

        constraints = list(model._meta.constraints)

        # Provide isolated instances of the fields to the new model body so
        # that the existing model's internals aren't interfered with when
        # the dummy model is constructed.
        body_copy = copy.deepcopy(body)

        # Construct a new model with the new fields to allow self referential
        # primary key to resolve to. This model won't ever be materialized as a
        # table and solely exists for foreign key reference resolution purposes.
        # This wouldn't be required if the schema editor was operating on model
        # states instead of rendered models.
        meta_contents = {
            'app_label': model._meta.app_label,
            'db_table': model._meta.db_table,
            'unique_together': unique_together,
            'index_together': index_together,
            'indexes': indexes,
            'constraints': constraints,
            'apps': apps,
        }
        meta = type("Meta", (), meta_contents)
        body_copy['Meta'] = meta
        body_copy['__module__'] = model.__module__
        type(model._meta.object_name, model.__bases__, body_copy)

        # Construct a model with a renamed table name.
        body_copy = copy.deepcopy(body)
        suffix = ''.join(choice(ascii_lowercase) for _ in range(6))
        meta_contents = {
            'app_label': model._meta.app_label,
            'db_table': 'new__%s__%s' % (suffix, strip_quotes(model._meta.db_table)),
            'unique_together': unique_together,
            'index_together': index_together,
            'indexes': indexes,
            'constraints': constraints,
            'apps': apps,
        }
        meta = type("Meta", (), meta_contents)
        body_copy['Meta'] = meta
        body_copy['__module__'] = model.__module__
        new_model = type('New%s' % model._meta.object_name, model.__bases__, body_copy)

        # Create a new table with the updated schema.
        self.create_model(new_model)

        # Copy data from the old table into the new table
        self.execute("INSERT INTO %s (%s) SELECT %s FROM %s" % (
            self.quote_name(new_model._meta.db_table),
            ', '.join(self.quote_name(x) for x in mapping),
            ', '.join(mapping.values()),
            self.quote_name(model._meta.db_table),
        ))

        # Delete the old table to make way for the new
        self.delete_model(model, handle_autom2m=False)

        # Rename the new table to take way for the old
        self.alter_db_table(
            new_model, new_model._meta.db_table, model._meta.db_table,
            # disable_constraints=False,
        )

        # Run deferred SQL on correct table
        for sql in self.deferred_sql:
            self.execute(sql)
        self.deferred_sql = []
        # Fix any PK-removed field
        if restore_pk_field:
            restore_pk_field.primary_key = True

    def delete_model(self, model, handle_autom2m=True):
        self.drop_referenced_fk(model)

        if handle_autom2m:
            super().delete_model(model)
        else:
            # Delete the table (and only that)
            self.execute(self.sql_delete_table % {
                "table": self.quote_name(model._meta.db_table),
            })
            # Remove all deferred statements referencing the deleted table.
            for sql in list(self.deferred_sql):
                if isinstance(sql, Statement) and sql.references_table(model._meta.db_table):
                    self.deferred_sql.remove(sql)

    def add_field(self, model, field):
        """
        Create a field on a model. Usually involves adding a column, but may
        involve adding a table instead (for M2M fields).
        """
        # Special-case implicit M2M tables
        if field.many_to_many and field.remote_field.through._meta.auto_created:
            return self.create_model(field.remote_field.through)
        self._remake_table(model, create_field=field)

    def remove_field(self, model, field):
        """
        Remove a field from a model. Usually involves deleting a column,
        but for M2Ms may involve deleting a table.
        """
        # M2M fields are a special case
        if field.many_to_many:
            # For implicit M2M tables, delete the auto-created table
            if field.remote_field.through._meta.auto_created:
                self.delete_model(field.remote_field.through)
            # For explicit "through" M2M fields, do nothing
        # For everything else, remake.
        else:
            # It might not actually have a column behind it
            if field.db_parameters(connection=self.connection)['type'] is None:
                return
            self._remake_table(model, delete_field=field)

    def _alter_field(self, model, old_field, new_field, old_type, new_type,
                     old_db_params, new_db_params, strict=False):
        """Perform a "physical" (non-ManyToMany) field update."""
        # Use "ALTER TABLE ... RENAME COLUMN" if only the column name
        # changed and there aren't any constraints.
        if (self.connection.features.can_alter_table_rename_column and
            old_field.column != new_field.column and
            self.column_sql(model, old_field) == self.column_sql(model, new_field) and
            not (old_field.remote_field and old_field.db_constraint or
                 new_field.remote_field and new_field.db_constraint)):
            return self.execute(self._rename_field_sql(model._meta.db_table, old_field, new_field, new_type))
        # Alter by remaking table
        self._remake_table(model, alter_field=(old_field, new_field))
        # Rebuild tables with FKs pointing to this field if the PK type changed.
        if old_field.primary_key and new_field.primary_key and old_type != new_type:
            for rel in new_field.model._meta.related_objects:
                if not rel.many_to_many:
                    self._remake_table(rel.related_model)

    def _alter_many_to_many(self, model, old_field, new_field, strict):
        """Alter M2Ms to repoint their to= endpoints."""
        if old_field.remote_field.through._meta.db_table == new_field.remote_field.through._meta.db_table:
            # The field name didn't change, but some options did; we have to propagate this altering.
            self._remake_table(
                old_field.remote_field.through,
                alter_field=(
                    # We need the field that points to the target model, so we can tell alter_field to change it -
                    # this is m2m_reverse_field_name() (as opposed to m2m_field_name, which points to our model)
                    old_field.remote_field.through._meta.get_field(old_field.m2m_reverse_field_name()),
                    new_field.remote_field.through._meta.get_field(new_field.m2m_reverse_field_name()),
                ),
            )
            return

        # Make a new through table
        self.create_model(new_field.remote_field.through)
        # Copy the data across
        self.execute("INSERT INTO %s (%s) SELECT %s FROM %s" % (
            self.quote_name(new_field.remote_field.through._meta.db_table),
            ', '.join([
                "id",
                new_field.m2m_column_name(),
                new_field.m2m_reverse_name(),
            ]),
            ', '.join([
                "id",
                old_field.m2m_column_name(),
                old_field.m2m_reverse_name(),
            ]),
            self.quote_name(old_field.remote_field.through._meta.db_table),
        ))
        # Delete the old through table
        self.delete_model(old_field.remote_field.through)

    def add_constraint(self, model, constraint):
        if isinstance(constraint, UniqueConstraint) and constraint.condition:
            super().add_constraint(model, constraint)
        else:
            self._remake_table(model)

    def remove_constraint(self, model, constraint):
        if isinstance(constraint, UniqueConstraint) and constraint.condition:
            super().remove_constraint(model, constraint)
        else:
            self._remake_table(model)

    def drop_referenced_fk(self, model):
        sql = """
            SELECT constraint_name, referencing FROM 
                (SELECT "name" AS constraint_name, 
                    (SELECT "name" 
                     FROM "_vspace" x 
                     WHERE x."id" = y."child_id") 
                     AS referencing, 
                        (SELECT "name" 
                         FROM "_vspace" x 
                         WHERE x."id" = y."parent_id") 
                     AS referenced 
            FROM "_fk_constraint" y where referenced = '%s')
        """ % model._meta.db_table
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            for constraint, table in cursor.rows:
                cursor.execute('ALTER TABLE "%s" DROP CONSTRAINT "%s"' % (table, constraint))


"""
The code has taken from django/db/backends/sqlite3/schema.py and slightly
reworked for Tarantool compatibility

Copyright (c) Django Software Foundation and individual contributors.
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

    3. Neither the name of Django nor the names of its contributors may be used
       to endorse or promote products derived from this software without
       specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""


import uuid

from django.conf import settings
from django.utils import timezone
from django.utils.datetime_safe import datetime
from django.db.backends.base.operations import BaseDatabaseOperations
from datetime import time


class DatabaseOperations(BaseDatabaseOperations):

    def quote_name(self, name):
        return '"%s"' % name

    def pk_default_value(self):
        return 'NULL'

    def get_db_converters(self, expression):
        converters = []
        internal_type = expression.output_field.get_internal_type()
        if internal_type == 'DateTimeField':
            converters.append(self.convert_datetimefield_value)
        elif internal_type == 'DateField':
            converters.append(self.convert_datefield_value)
        elif internal_type == 'TimeField':
            converters.append(self.convert_timefield_value)
        elif internal_type == 'UUIDField':
            converters.append(self.convert_uuidfield_value)
        elif internal_type in ('BooleanField', 'NullBooleanField'):
            converters.append(self.convert_booleanfield_value)
        return converters

    def convert_booleanfield_value(self, value, expression, connection):
        return value

    def convert_datefield_value(self, value, expression, connection):
        if value is None:
            return None
        return datetime.fromtimestamp(int(value)).date()

    def convert_datetimefield_value(self, value, expression, connection):
        if value is None:
            return None
        value = datetime.fromtimestamp(value)
        if settings.USE_TZ:
            value = timezone.make_aware(value, self.connection.timezone)
        return value

    def convert_timefield_value(self, value, expression, connection):
        if value is None:
            return None
        hours, _seconds = divmod(value, 3600)
        minutes, seconds = divmod(_seconds, 60)
        return time(hours, minutes, seconds)

    def convert_uuidfield_value(self, value, expression, connection):
        if value is not None:
            value = uuid.UUID(value)
        return value

    def adapt_datetimefield_value(self, value):
        return value.timestamp() if value else None

    def adapt_datefield_value(self, value):
        return datetime.fromordinal(value.toordinal()).timestamp() if value is not None else None

    def adapt_timefield_value(self, value):
        return value.hour * 60 * 60 + value.minute * 60 + value.second if \
            value is not None else None

    def lookup_cast(self, lookup_type, internal_type=None):
        if lookup_type in ('iexact', 'icontains', 'istartswith', 'iendswith'):
            return 'UPPER(%s)'
        return '%s'

    def bulk_insert_sql(self, fields, placeholder_rows):
        placeholder_rows_sql = (", ".join(row) for row in placeholder_rows)
        values_sql = ", ".join("(%s)" % sql for sql in placeholder_rows_sql)
        return "VALUES " + values_sql

    def bulk_batch_size(self, fields, objs):
        if fields:
            return self.connection.features.max_query_params // len(fields)
        return len(objs)

    def no_limit_value(self):
        return 131072

    def sql_flush(self, style, tables, *args, **kwargs):
        return []



import datetime

from django.core.exceptions import ValidationError
from django.test import TestCase
from django.utils import timezone

from app.models import Time


class TimeTests(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.morning_time_value = datetime.time(hour=9, minute=20, second=10)
        cls.evening_time_value = datetime.time(hour=19, minute=2, second=55)
        cls.time_value = timezone.now().time().replace(microsecond=0)
        Time.objects.create(time=cls.time_value)
        Time.objects.create(time=cls.morning_time_value)
        Time.objects.create(time=cls.evening_time_value)

    def test_time_values(self):
        time_item = Time.objects.get(time=self.time_value)
        self.assertEqual(time_item.time, self.time_value)
        time_now = timezone.now().time()
        self.assertLess(time_item.time, time_now)
        self.assertGreater(time_now, time_item.time)

    def test_lookups(self):
        time_now = timezone.now().time()
        Time.objects.filter(time__lte=time_now)
        self.assertTrue(Time.objects.filter(time__lt=self.evening_time_value).exists())
        self.assertTrue(Time.objects.filter(time__lte=self.evening_time_value).exists())
        self.assertTrue(Time.objects.filter(time__gt=self.morning_time_value).exists())
        self.assertTrue(Time.objects.filter(time__gte=self.morning_time_value).exists())

    def test_time_invalid_value(self):
        with self.assertRaises(ValidationError):
            Time.objects.create(time='string value')



from django.db import DatabaseError
from django.test import TestCase

from app.models import PositiveSmallInteger


class PositiveSmallIntegerTests(TestCase):
    def setUp(self):
        self.int0_id = PositiveSmallInteger.objects.create(positive_small_integer=0).id
        self.int1_id = PositiveSmallInteger.objects.create(positive_small_integer=1111).id

    def test_create_integer(self):
        int0 = PositiveSmallInteger.objects.get(id=self.int0_id)
        int1 = PositiveSmallInteger.objects.get(id=self.int1_id)
        self.assertEqual(int0.positive_small_integer, 0)
        self.assertEqual(int1.positive_small_integer, 1111)
        self.assertLess(int0.positive_small_integer, int1.positive_small_integer)
        self.assertGreater(int1.positive_small_integer, int0.positive_small_integer)

    def test_extremal_values(self):
        int_biggest = PositiveSmallInteger.objects.create(positive_small_integer=18446744073709551615)
        self.assertEqual(int_biggest.positive_small_integer, 18446744073709551615)
        int_smallest = PositiveSmallInteger.objects.create(positive_small_integer=0)
        self.assertEqual(int_smallest.positive_small_integer, 0)
        self.assertLess(int_smallest.positive_small_integer, int_biggest.positive_small_integer)
        with self.assertRaises(Exception):
            PositiveSmallInteger.objects.create(
                positive_small_integer=18446744073709551616)
        with self.assertRaises(DatabaseError):
            PositiveSmallInteger.objects.create(positive_small_integer=-1)



# Generated by Django 3.0.7 on 2020-06-27 09:22

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BigInteger',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('big_integer', models.BigIntegerField()),
                ('integer', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='Binary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('binary', models.BinaryField()),
            ],
        ),
        migrations.CreateModel(
            name='Boolean',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('boolean', models.BooleanField(default=False, verbose_name='Bool value')),
            ],
        ),
        migrations.CreateModel(
            name='Char',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('char', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Date',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField()),
            ],
        ),
        migrations.CreateModel(
            name='DateTime',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_time', models.DateTimeField(blank=True, null=True)),
                ('date_time_auto', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Decimal',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('decimal', models.DecimalField(decimal_places=2, max_digits=5)),
            ],
        ),
        migrations.CreateModel(
            name='Duration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('duration', models.DurationField()),
            ],
        ),
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='')),
            ],
        ),
        migrations.CreateModel(
            name='FilePath',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filepath', models.FilePathField(path='/Users/artem.morozov/projects/django-tarantool')),
            ],
        ),
        migrations.CreateModel(
            name='Float',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('float', models.FloatField()),
            ],
        ),
        migrations.CreateModel(
            name='GenericIPAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip_address_v4', models.GenericIPAddressField(protocol='ipv4')),
                ('ip_address_v6', models.GenericIPAddressField(protocol='ipv6')),
                ('generic_ip_address', models.GenericIPAddressField()),
            ],
        ),
        migrations.CreateModel(
            name='Integer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('integer', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='M2MDependency',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='NullBoolean',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('null_boolean', models.NullBooleanField(verbose_name='Null bool value')),
            ],
        ),
        migrations.CreateModel(
            name='OneToOneRelative',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='PositiveInteger',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('positive_integer', models.PositiveIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='PositiveSmallInteger',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('positive_small_integer', models.PositiveSmallIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='Slug',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField()),
            ],
        ),
        migrations.CreateModel(
            name='SmallInteger',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('small_integer', models.SmallIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='Text',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Time',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time', models.TimeField()),
            ],
        ),
        migrations.CreateModel(
            name='Uuid',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField()),
            ],
        ),
        migrations.CreateModel(
            name='OneToOne',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('one_to_one', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='one', to='app.OneToOneRelative')),
            ],
        ),
        migrations.CreateModel(
            name='ManyToMany',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=20)),
                ('m2m', models.ManyToManyField(blank=True, to='app.M2MDependency')),
            ],
        ),
        migrations.CreateModel(
            name='ForeignKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('foreign_key', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='app.Integer')),
            ],
        ),
        migrations.CreateModel(
            name='AllModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('boolean', models.BooleanField(default=True)),
                ('char', models.CharField(max_length=1)),
                ('decimal', models.DecimalField(decimal_places=2, max_digits=5)),
                ('file', models.FileField(upload_to='')),
                ('filepath', models.FilePathField(path='/Users/artem.morozov/projects/django-tarantool')),
                ('float', models.FloatField(null=True)),
                ('integer', models.IntegerField(null=True)),
                ('big_integer', models.BigIntegerField(null=True)),
                ('generic_ip_address', models.GenericIPAddressField()),
                ('null_boolean', models.NullBooleanField()),
                ('positive_integer', models.PositiveIntegerField()),
                ('positive_small_integer', models.PositiveSmallIntegerField()),
                ('slug', models.SlugField()),
                ('small_integer', models.SmallIntegerField()),
                ('text', models.TextField(null=True)),
                ('time', models.TimeField()),
                ('uuid', models.UUIDField(default=uuid.uuid4)),
                ('fk', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='all_models', to='app.Integer')),
                ('one_to_one', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='all_model', to='app.Integer')),
            ],
        ),
    ]



from datetime import timedelta

from django.test import TestCase
from django.utils import timezone

from app.models import Date


class DateTests(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.date_value = timezone.now().date()
        cls.date_in_the_future = cls.date_value + timedelta(days=10)
        cls.date_in_the_past = cls.date_value - timedelta(days=10)
        Date.objects.create(date=cls.date_value)
        Date.objects.create(date=cls.date_in_the_future)
        Date.objects.create(date=cls.date_in_the_past)

    def test_date(self):
        date = Date.objects.get(date=self.date_value)
        self.assertEqual(self.date_value, date.date)

    def test_date_lt(self):
        dates = Date.objects.filter(date__lt=self.date_in_the_future)
        self.assertEqual(dates.count(), 2)

    def test_date_gt(self):
        dates = Date.objects.filter(date__gt=self.date_in_the_past)
        self.assertEqual(dates.count(), 2)

    def test_date_gte(self):
        dates = Date.objects.filter(date__gte=self.date_value)
        self.assertEqual(dates.count(), 2)

    def test_date_lte(self):
        dates = Date.objects.filter(date__gte=self.date_value)
        self.assertEqual(dates.count(), 2)



from django.test import TestCase
from django.utils import lorem_ipsum

from app.models import Text


class TextTests(TestCase):
    @classmethod
    def setUpClass(cls):
        text_value = lorem_ipsum.words(100)
        cls.text_value = text_value
        Text.objects.create(text=text_value)
        cls.end = ' '.join(text_value.split()[-2:])

    def test_text_get(self):
        text_item = Text.objects.get(text=self.text_value)
        self.assertEqual(text_item.text, self.text_value)
        with self.assertRaises(Text.DoesNotExist):
            Text.objects.get(text='string which is not in the text')

    def test_text_contains(self):
        self.assertTrue(Text.objects.filter(text__contains='ipsum').exists())
        self.assertFalse(Text.objects.filter(text__contains='string which is not in the text').exists())

    def test_text_icontains(self):
        self.assertTrue(Text.objects.filter(text__icontains='IpSuM').exists())
        self.assertFalse(Text.objects.filter(text__icontains='string which is not in the text').exists())

    def test_text_exact(self):
        self.assertTrue(Text.objects.filter(text__exact=self.text_value).exists())
        self.assertFalse(Text.objects.filter(text__exact='string which is not in the text').exists())

    def test_text_iexact(self):
        self.assertTrue(Text.objects.filter(text__iexact=self.text_value.upper()).exists())
        self.assertFalse(Text.objects.filter(text__iexact='string which is not in the text').exists())

    def test_text_startswith(self):
        self.assertTrue(Text.objects.filter(text__startswith='lorem ipsum').exists())
        self.assertFalse(Text.objects.filter(text__startswith='string which is not in the text').exists())

    def test_text_istartswith(self):
        self.assertTrue(Text.objects.filter(text__istartswith='lOreM iPSUm').exists())
        self.assertFalse(Text.objects.filter(text__istartswith='string which is not in the text').exists())

    def test_text_endswith(self):
        self.assertTrue(Text.objects.filter(text__endswith=self.end).exists())
        self.assertFalse(Text.objects.filter(text__endswith='string which is not in the text').exists())

    def test_text_iendswith(self):
        self.assertTrue(Text.objects.filter(text__iendswith=self.end.upper()).exists())
        self.assertFalse(Text.objects.filter(text__iendswith='string which is not in the text').exists())



from django.db import DatabaseError
from django.test import TestCase

from app.models import BigInteger


class BigIntegerTests(TestCase):
    def setUp(self):
        self.int0_id = BigInteger.objects.create(big_integer=0).id
        self.int1_id = BigInteger.objects.create(big_integer=1111).id

    def test_create_integer(self):
        int0 = BigInteger.objects.get(id=self.int0_id)
        int1 = BigInteger.objects.get(id=self.int1_id)
        self.assertEqual(int0.big_integer, 0)
        self.assertEqual(int1.big_integer, 1111)
        self.assertLess(int0.big_integer, int1.big_integer)
        self.assertGreater(int1.big_integer, int0.big_integer)

    def test_extremal_values(self):
        int_biggest = BigInteger.objects.create(big_integer=18446744073709551615)
        self.assertEqual(int_biggest.big_integer, 18446744073709551615)
        int_smallest = BigInteger.objects.create(big_integer=-9223372036854775808)
        self.assertEqual(int_smallest.big_integer, -9223372036854775808)
        self.assertLess(int_smallest.big_integer, int_biggest.big_integer)
        with self.assertRaises(Exception):
            BigInteger.objects.create(big_integer=18446744073709551616)
        with self.assertRaises(Exception):
            BigInteger.objects.create(big_integer=-9223372036854776840)



import os
import shutil
import subprocess
from pathlib import Path

from django.core.management import call_command
from django.db.backends.base.creation import BaseDatabaseCreation

from django_app import settings
from django_tarantool.backend.utils import wait_for_tarantool

START_LUA = """\
box.cfg({ listen = %d })
box.schema.user.passwd('admin', 'password')
"""


class DatabaseCreation(BaseDatabaseCreation):
    tarantool_proc = None
    tarantool_test_port = 3302
    db_path = None

    def _quote_name(self, name):
        return self.connection.ops.quote_name(name)

    def _prepare_db_path(self, db_name):
        path = os.path.join(settings.BASE_DIR, db_name)
        Path(path).mkdir(parents=True, exist_ok=True)
        with open(os.path.join(path, 'start.lua'), 'w+') as file:
            file.write(START_LUA % self.tarantool_test_port)
        return path

    def start_test_tarantool(self):
        test_database_name = self._get_test_db_name()
        self.db_path = self._prepare_db_path(test_database_name)
        self.tarantool_proc = subprocess.Popen(['tarantool', 'start.lua'], cwd=self.db_path)
        wait_for_tarantool(self.connection)

    def create_test_db(self, verbosity=1, autoclobber=False, serialize=True, keepdb=False):
        settings.DATABASES[self.connection.alias]["PORT"] = self.tarantool_test_port
        self.connection.settings_dict["PORT"] = self.tarantool_test_port
        self.start_test_tarantool()

        call_command('migrate', 'app', interactive=False)

    def destroy_test_db(self, old_database_name=None, verbosity=1, keepdb=False, suffix=None):
        self.tarantool_proc.kill()
        super().destroy_test_db(old_database_name, verbosity, keepdb, suffix)

    def _destroy_test_db(self, test_database_name, verbosity):
        shutil.rmtree(self.db_path)



# from django.db import DatabaseError
# from django.test import TestCase
#
# from app.models import ManyToMany, ForeignKey
#
#
# class ManyToManyTests(TestCase):
#     @classmethod
#     def setUpClass(cls):
#         for _ in range(10):
#             fk = ForeignKey.objects.create()
#             m2m = ManyToMany.objects.create(name='m2m_initial')



from django.db import DatabaseError
from django.test import TestCase

from app.models import Float


class FloatTests(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.float_value = 44.66
        cls.float_item_id = Float.objects.create(float=cls.float_value).id

    def test_float_create(self):
        float_item = Float.objects.get(id=self.float_item_id)
        self.assertEqual(float_item.float, self.float_value)

    def test_invalid_float(self):
        with self.assertRaises(ValueError):
            Float.objects.create(float='string value')
        with self.assertRaises(DatabaseError):  # Why DatabaseError?
            Float.objects.create(float=None)



# from io import BytesIO
#
# from django.core.exceptions import ValidationError
# from django.test import TestCase
#
# from app.models import Binary
# from django.db import DatabaseError
#
#
# class BinaryTests(TestCase):
#
#     @classmethod
#     def setUpClass(cls):
#         # cls.bytes = BytesIO(b'Binary content')
#         test_string = "Binary content"
#         res = bytes(test_string, 'utf-8')
#         Binary.objects.create(binary=res)
#
#     def test_binary_content(self):
#         binary = Binary.objects.get()
#         print(binary)



from .boolean import *
from .null_boolean import *
from .datetime import *
from .time import *
from .date import *
from .float import *
from .decimal import *
from .integer import *
from .big_integer import *
from .positive_integer import *
from .small_integer import *
from .positive_small_integer import *
from .char import *
from .text import *
from .uuid import *
from .slug import *
from .one_to_one import *
from .foreign_key import *
from .binary import *
from .many_to_many import *
from .generic_ip_address import *



from django.contrib import admin

from . import models


admin.site.register(models.AllModel)
admin.site.register(models.Float)
admin.site.register(models.Decimal)
admin.site.register(models.BigInteger)
admin.site.register(models.Integer)
admin.site.register(models.SmallInteger)
admin.site.register(models.PositiveInteger)
admin.site.register(models.PositiveSmallInteger)
admin.site.register(models.Boolean)
admin.site.register(models.NullBoolean)

@admin.register(models.DateTime)
class DateTimeAdmin(admin.ModelAdmin):
    readonly_fields = ('date_time_auto',)

admin.site.register(models.Date)
admin.site.register(models.Time)


@admin.register(models.Char)
class CharAdmin(admin.ModelAdmin):
    list_per_page = 5


admin.site.register(models.Text)
admin.site.register(models.GenericIPAddress)
admin.site.register(models.File)
admin.site.register(models.FilePath)
admin.site.register(models.OneToOne)


@admin.register(models.OneToOneRelative)
class OneToOneRelative(admin.ModelAdmin):
    readonly_fields = ('id',)



"""
Tarantool database backend for Django.
"""


from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.utils import (
    CursorDebugWrapper as BaseCursorDebugWrapper,
)

from .client import DatabaseClient                          # NOQA isort:skip
from .creation import DatabaseCreation                      # NOQA isort:skip
from .features import DatabaseFeatures                      # NOQA isort:skip
from .introspection import DatabaseIntrospection            # NOQA isort:skip
from .operations import DatabaseOperations                  # NOQA isort:skip
from .schema import DatabaseSchemaEditor                    # NOQA isort:skip


from tarantool import dbapi


Database = dbapi


class DatabaseWrapper(BaseDatabaseWrapper):
    def is_usable(self):
        return True

    vendor = 'tarantool'
    display_name = 'Tarantool'
    Database = Database
    SchemaEditorClass = DatabaseSchemaEditor
    client_class = DatabaseClient
    creation_class = DatabaseCreation
    features_class = DatabaseFeatures
    introspection_class = DatabaseIntrospection
    ops_class = DatabaseOperations

    data_types = {
        'AutoField': 'INTEGER',
        'BigAutoField': 'INTEGER',
        'BinaryField': 'VARBINARY',
        'BooleanField': 'BOOLEAN',
        'NullBooleanField': 'BOOLEAN',
        'CharField': 'VARCHAR(%(max_length)s)',
        'TextField': 'TEXT',
        'UUIDField': 'VARCHAR(32)',
        'SlugField': 'VARCHAR(%(max_length)s)',
        'DateField': 'UNSIGNED',
        'DateTimeField': 'NUMBER',
        'TimeField': 'UNSIGNED',
        'DurationField': 'INTEGER',
        'OneToOneField': 'INTEGER',
        'FileField': 'VARCHAR(%(max_length)s)',
        'FilePathField': 'VARCHAR(%(max_length)s)',
        'BigIntegerField': 'INTEGER',
        'IntegerField': 'INTEGER',
        'PositiveIntegerField': 'UNSIGNED',
        'SmallIntegerField': 'INTEGER',
        'PositiveSmallIntegerField': 'UNSIGNED',
        'FloatField': 'NUMBER',
        'DecimalField': 'NUMBER',
        'IPAddressField': 'VARCHAR(15)',
        'GenericIPAddressField': 'VARCHAR(39)',
    }

    data_types_suffix = {
        'AutoField': 'AUTOINCREMENT',
        'BigAutoField': 'AUTOINCREMENT',
    }

    pattern_esc = r"REPLACE(REPLACE(REPLACE({}, '\', '\\'), '%%', '\%%'), '_', '\_')"
    operators = {
        'exact': '= %s',
        'iexact': "= UPPER(%s)",
        'contains': "LIKE %s",
        'icontains': "LIKE UPPER(%s)",
        'regex': 'REGEXP %s',
        'iregex': "REGEXP '(?i)' || %s",
        'gt': '> %s',
        'gte': '>= %s',
        'lt': '< %s',
        'lte': '<= %s',
        'startswith': "LIKE %s",
        'endswith': "LIKE %s",
        'istartswith': "LIKE UPPER(%s)",
        'iendswith': "LIKE UPPER(%s)",
    }

    pattern_ops = {
        'contains': r"LIKE '%%' || {} || '%%' ESCAPE '\'",
        'icontains': r"LIKE '%%' || UPPER('{}') || '%%' ESCAPE '\'",
        'startswith': r"LIKE {} || '%%' ESCAPE '\'",
        'istartswith': r"LIKE UPPER({}) || '%%' ESCAPE '\'",
        'endswith': r"LIKE '%%' || {} ESCAPE '\'",
        'iendswith': r"LIKE '%%' || UPPER({}) ESCAPE '\'",
    }

    def get_connection_params(self):
        settings_dict = self.settings_dict

        conn_params = {
            **settings_dict['OPTIONS'],
        }
        conn_params.pop('isolation_level', None)
        if settings_dict['USER']:
            conn_params['user'] = settings_dict['USER']
        if settings_dict['PASSWORD']:
            conn_params['password'] = settings_dict['PASSWORD']
        if settings_dict['HOST']:
            conn_params['host'] = settings_dict['HOST']
        if settings_dict['PORT']:
            conn_params['port'] = settings_dict['PORT']
        conn_params['use_list'] = False
        return conn_params

    def get_new_connection(self, conn_params):
        connection = dbapi.connect(**conn_params)
        return connection

    def _set_autocommit(self, value):
        pass

    def init_connection_state(self):
        pass

    def create_cursor(self, name=None):
        return CursorWrapper(self.connection.cursor())

    def make_debug_cursor(self, cursor):
        return CursorDebugWrapper(cursor, self)


class CursorWrapper:
    """Tarantool supports only qmark and named param styles
    the easies way to send SQL query with proper for tarantool
    parameters is to convert them to qmark style.
    We need to overwrite execute and executemany methods to do it.
    """
    def __init__(self, cursor):
        self.cursor = cursor

    def execute(self, query, params=()):
        query = self.convert_query(query, len(params)) if params else query
        return self.cursor.execute(query, params)

    def executemany(self, query, param_list):
        query = self.convert_query(query, len(param_list[0])) if param_list else query
        return self.cursor.executemany(query, param_list)

    def convert_query(self, query, num_params):
        return query % tuple("?" * num_params)

    def __getattr__(self, attr):
        return getattr(self.cursor, attr)

    def __iter__(self):
        return iter(self.cursor)


class CursorDebugWrapper(BaseCursorDebugWrapper):
    def copy_expert(self, sql, file, *args):
        with self.debug_sql(sql):
            return self.cursor.copy_expert(sql, file, *args)

    def copy_to(self, file, table, *args, **kwargs):
        with self.debug_sql(sql='COPY %s TO STDOUT' % table):
            return self.cursor.copy_to(file, table, *args, **kwargs)



from random import randint

from django.test import TestCase

from app.models import ForeignKey, Integer


class ForeignKeyTests(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.random_int = randint(1, 1000000)
        integer = Integer.objects.create(integer=cls.random_int)
        for _ in range(10):
            ForeignKey.objects.create(foreign_key=integer)

    def test_foreign_key(self):
        integer = Integer.objects.get(integer=self.random_int)
        fk = ForeignKey.objects.filter(foreign_key=integer)[0]
        self.assertEqual(fk.foreign_key, integer)

    def test_foreign_key_count(self):
        integer = Integer.objects.get(integer=self.random_int)
        self.assertEqual(ForeignKey.objects.count(), integer.foreignkey_set.count())



from django.test import TestCase

from app.models import GenericIPAddress


class ForeignKeyTests(TestCase):

    def test_create_ip_addresses(self):
        GenericIPAddress.objects.create(
            ip_address_v4='127.0.0.1',
            ip_address_v6='2001:0db8:0000:0000:0000:ff00:0042:8329',
            generic_ip_address='192.168.1.1',
        )
        self.assertEqual(
            GenericIPAddress.objects.get(ip_address_v4='127.0.0.1').ip_address_v4,
            '127.0.0.1')
        self.assertEqual(
            GenericIPAddress.objects.get(ip_address_v6='2001:0db8:0000:0000:0000:ff00:0042:8329').ip_address_v6,
            '2001:db8::ff00:42:8329')
        self.assertEqual(
            GenericIPAddress.objects.get(generic_ip_address='192.168.1.1').generic_ip_address,
            '192.168.1.1')

        GenericIPAddress.objects.create(
            ip_address_v4='0.0.0.0',
            ip_address_v6='2001:db8:0:0:0:ff00:42:8339',
            generic_ip_address='255.255.128.0',
        )
        self.assertEqual(
            GenericIPAddress.objects.get(ip_address_v4='0.0.0.0').ip_address_v4,
            '0.0.0.0')
        self.assertEqual(
            GenericIPAddress.objects.get(ip_address_v6='2001:db8:0:0:0:ff00:42:8339').ip_address_v6,
            '2001:db8::ff00:42:8339')
        self.assertEqual(
            GenericIPAddress.objects.get(generic_ip_address='255.255.128.0').generic_ip_address,
            '255.255.128.0')

        GenericIPAddress.objects.create(
            ip_address_v4='255.255.255.255',
            ip_address_v6='2001:4860:4861::8888',
            generic_ip_address='172.16.10.10',
        )
        self.assertEqual(
            GenericIPAddress.objects.get(ip_address_v4='255.255.255.255').ip_address_v4,
            '255.255.255.255')
        self.assertEqual(
            GenericIPAddress.objects.get(ip_address_v6='2001:4860:4861::8888').ip_address_v6,
            '2001:4860:4861::8888')
        self.assertEqual(
            GenericIPAddress.objects.get(generic_ip_address='172.16.10.10').generic_ip_address,
            '172.16.10.10')



from collections import namedtuple

from django.db.backends.base.introspection import BaseDatabaseIntrospection, \
    TableInfo, FieldInfo as BaseFieldInfo


FieldInfo = namedtuple('FieldInfo', BaseFieldInfo._fields + ('pk',))


class DatabaseIntrospection(BaseDatabaseIntrospection):
    data_types_reverse = {
        'bool': 'BooleanField',
        'boolean': 'BooleanField',
        'integer': 'IntegerField',
        'number': 'FloatField',
        'unsigned': 'PositiveIntegerField',
        'decimal': 'DecimalField',
        'text': 'TextField',
        'string': 'CharField',
        'varchar': 'CharField',
    }

    def get_field_type(self, data_type, description):
        field_type = super().get_field_type(data_type, description)
        if description.pk and field_type in {'BigIntegerField', 'IntegerField',
                                             'SmallIntegerField'}:
            return 'AutoField'
        return field_type

    def get_constraints(self, cursor, table_name):
        """
        Retrieve any constraints or keys (unique, pk, fk, check, index)
        across one or more columns.

        Return a dict mapping constraint names to their attributes,
        where attributes is a dict with keys:
         * columns: List of columns this covers
         * primary_key: True if primary key, False otherwise
         * unique: True if this is a unique constraint, False otherwise
         * foreign_key: (table, column) of target, or None
         * check: True if check constraint, False otherwise
         * index: True if index, False otherwise.
         * orders: The order (ASC/DESC) defined for the columns of indexes
         * type: The type of the index (btree, hash, etc.)
        """

        constraints = {}
        cursor.execute("""
            SELECT
                "name" AS constraint_name,
                (SELECT "name" FROM "_vspace" x WHERE x."id" = y."id") AS 
                table_name,
                CASE WHEN "iid" = 0 THEN 'PRIMARY' ELSE 'UNIQUE' END AS 
                constraint_type,
                "id" AS id,
                "iid" AS iid,
                "type" AS index_type
            FROM "_vindex" y
            WHERE table_name = %s;""", [table_name])

        for index, table_name, constraint_type, _id, iid, index_type \
                in cursor.fetchall():

            constraints[index] = {
                "columns": [],
                "primary_key": constraint_type == 'PRIMARY',
                "unique": constraint_type == 'UNIQUE',
                "foreign_key": None,
                "check": False,
                "index": False,
            }
            cursor.execute('PRAGMA index_info(%s.%s)' % (
                self.connection.ops.quote_name(table_name),
                self.connection.ops.quote_name(index)
            ))

            for constraint_data in cursor.fetchall():
                constraints[index]['columns'].append(constraint_data[2])

        cursor.execute(
            'PRAGMA foreign_key_list(%s)' % self.connection.ops.quote_name(
                table_name))
        for row in cursor.fetchall():
            # Remaining on_update/on_delete/match values are of no interest.
            id_, _, table, from_, to = row[:5]
            constraints['fk_%d' % id_] = {
                'columns': [from_],
                'primary_key': False,
                'unique': False,
                'foreign_key': (table, to),
                'check': False,
                'index': False,
            }
        cursor.execute("""
        SELECT
            "name" AS constraint_name,
            "code" AS check_clause,
            (SELECT "name" FROM "_vspace" x WHERE x."id" = y."space_id") AS 
            table_name,
            "language" AS language,
            "is_deferred" AS is_deferred,
            "space_id" AS space_id
        FROM "_ck_constraint" y
        WHERE table_name = %s;""", [table_name])
        for row in cursor.fetchall():
            constraint_name = row[0]
            constraints[constraint_name] = {
                'columns': [],
                'primary_key': False,
                'unique': False,
                'foreign_key': None,
                'check': True,
                'index': False,
            }
        return constraints

    def get_relations(self, cursor, table_name):
        """
        Return a dictionary of {field_name: (field_name_other_table,
        other_table)} representing all relationships to the given table.
        """
        # Dictionary of relations to return
        relations = {
            key[0]: (key[2], key[1])
            for key in self.get_key_columns(cursor, table_name)
        }

        return relations

    def get_table_description(self, cursor, table_name):
        """
        Return a description of the table with the DB-API cursor.description
        interface.
        """
        cursor.execute('PRAGMA table_info(%s)' % self.connection.ops.quote_name(
            table_name))
        return [
            FieldInfo(
                name, data_type, None, None, None, None,
                not notnull, default, pk == 1,
            )
            for cid, name, data_type, notnull, default, pk in cursor.fetchall()
        ]

    def get_key_columns(self, cursor, table_name):
        """
        Return a list of (column_name, referenced_table_name,
        referenced_column_name)
        for all key columns in given table.
        """
        cursor.execute(
            'PRAGMA foreign_key_list(%s)' % self.connection.ops.quote_name(
                table_name))
        key_columns = []
        for _, _, referenced_table_name, column_name, referenced_column_name,\
            _, _, _ in cursor.fetchall():
            key_columns.append(
                [column_name, referenced_table_name, referenced_column_name])
        return key_columns

    def get_sequences(self, cursor, table_name, table_fields=()):
        """
        Return a list of introspected sequences for table_name. Each sequence
        is a dict: {'table': <table_name>, 'column': <column_name>}. An optional
        'name' key can be added if the backend supports named sequences.
        """
        pk_col = self.get_primary_key_column(cursor, table_name)
        return [{'table': table_name, 'column': pk_col}]

    def get_table_list(self, cursor):
        cursor.execute('SELECT "name" FROM "_space" '
                       'WHERE "name" NOT LIKE \'X_%\' ESCAPE \'X\'')

        return [TableInfo(table[0], 't') for table in cursor.fetchall()]



from django.test import TestCase
from django.utils import lorem_ipsum

from app.models import Slug


class SlugTests(TestCase):
    @classmethod
    def setUpClass(cls):
        slug_value = lorem_ipsum.words(100)
        cls.slug_value = slug_value
        Slug.objects.create(slug=slug_value)
        cls.end = ' '.join(slug_value.split()[-2:])

    def test_slug_get(self):
        slug_item = Slug.objects.get(slug=self.slug_value)
        self.assertEqual(slug_item.slug, self.slug_value)
        with self.assertRaises(Slug.DoesNotExist):
            Slug.objects.get(slug='string which is not in the slug')

    def test_slug_contains(self):
        self.assertTrue(Slug.objects.filter(slug__contains='ipsum').exists())
        self.assertTrue(Slug.objects.filter(slug__contains='lorem ipsum').exists())
        self.assertFalse(Slug.objects.filter(slug__contains='string which is not in the slug').exists())

    def test_slug_icontains(self):
        self.assertTrue(Slug.objects.filter(slug__icontains='IpSuM').exists())
        self.assertFalse(Slug.objects.filter(slug__icontains='string which is not in the slug').exists())

    def test_slug_exact(self):
        self.assertTrue(Slug.objects.filter(slug__exact=self.slug_value).exists())
        self.assertFalse(Slug.objects.filter(slug__exact='string which is not in the slug').exists())

    def test_slug_iexact(self):
        self.assertTrue(Slug.objects.filter(slug__iexact=self.slug_value.upper()).exists())
        self.assertFalse(Slug.objects.filter(slug__iexact='string which is not in the slug').exists())

    def test_slug_startswith(self):
        self.assertTrue(Slug.objects.filter(slug__startswith='lorem ipsum').exists())
        self.assertFalse(Slug.objects.filter(slug__startswith='string which is not in the slug').exists())

    def test_slug_istartswith(self):
        self.assertTrue(Slug.objects.filter(slug__istartswith='lOreM iPSUm').exists())
        self.assertFalse(Slug.objects.filter(slug__istartswith='string which is not in the slug').exists())

    def test_slug_endswith(self):
        self.assertTrue(Slug.objects.filter(slug__endswith=self.end).exists())
        self.assertFalse(Slug.objects.filter(slug__endswith='string which is not in the slug').exists())

    def test_slug_iendswith(self):
        self.assertTrue(Slug.objects.filter(slug__iendswith=self.end.upper()).exists())
        self.assertFalse(Slug.objects.filter(slug__iendswith='string which is not in the slug').exists())



from django.db import DatabaseError
from django.test import TestCase

from app.models import PositiveInteger


class PositiveIntegerTests(TestCase):
    def setUp(self):
        self.int0_id = PositiveInteger.objects.create(positive_integer=0).id
        self.int1_id = PositiveInteger.objects.create(positive_integer=1111).id

    def test_create_integer(self):
        int0 = PositiveInteger.objects.get(id=self.int0_id)
        int1 = PositiveInteger.objects.get(id=self.int1_id)
        self.assertEqual(int0.positive_integer, 0)
        self.assertEqual(int1.positive_integer, 1111)
        self.assertLess(int0.positive_integer, int1.positive_integer)
        self.assertGreater(int1.positive_integer, int0.positive_integer)

    def test_extremal_values(self):
        int_biggest = PositiveInteger.objects.create(positive_integer=18446744073709551615)
        self.assertEqual(int_biggest.positive_integer, 18446744073709551615)
        int_smallest = PositiveInteger.objects.create(positive_integer=0)
        self.assertEqual(int_smallest.positive_integer, 0)
        self.assertLess(int_smallest.positive_integer, int_biggest.positive_integer)
        with self.assertRaises(Exception):
            PositiveInteger.objects.create(
                positive_integer=18446744073709551616)
        with self.assertRaises(DatabaseError):
            PositiveInteger.objects.create(positive_integer=-1)



import os


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'kdtvq!t_(#lke93^r!b(c1a&q!c64vp2$*3il3-+nw0^u8=k9q'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']

INSTALLED_APPS = [
    'app',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'django_app.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases


DB_T = {
    'default': {
        'ENGINE': 'django_tarantool.backend',
        'HOST': '127.0.0.1',
        'PORT': '3301',
        'USER': 'admin',
        'PASSWORD': 'password',
        'CONN_MAX_AGE': 3600,
    }
}

DB_S = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'sqlite.db'),
    }
}

DB_P = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'postgres',
        'USER': 'postgres',
        'PASSWORD': 'parol',
        'HOST': 'localhost',
        'CONN_MAX_AGE': 3600,
    }
}

DATABASES = DB_T

# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'Europe/Moscow'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = '/static/'


LOGGING_TO_CONSOLE = {
    'version': 1,
    'filters': {
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue',
        }
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'filters': ['require_debug_true'],
            'class': 'logging.StreamHandler',
        }
    },
    'loggers': {
        'django.db.backends': {
            'level': 'DEBUG',
            'handlers': ['console'],
        }
    }
}

# LOGGING = LOGGING_TO_CONSOLE



import uuid as uuid
from django.conf import settings
from django.db import models


class Boolean(models.Model):
    boolean = models.BooleanField('Bool value', default=False)

    def __str__(self): return str(self.boolean)


class NullBoolean(models.Model):
    null_boolean = models.NullBooleanField('Null bool value')

    def __str__(self): return str(self.null_boolean)


class File(models.Model):
    file = models.FileField()

    def __str__(self): return self.file.path if self.file else None


class FilePath(models.Model):
    filepath = models.FilePathField(path=settings.BASE_DIR)

    def __str__(self): return self.filepath


class Float(models.Model):
    float = models.FloatField()

    def __str__(self): return str(self.float)


class Decimal(models.Model):
    decimal = models.DecimalField(max_digits=5, decimal_places=2)

    def __str__(self): return str(self.decimal)


class Integer(models.Model):
    integer = models.IntegerField()

    def __str__(self): return str(self.integer)


class BigInteger(models.Model):
    big_integer = models.BigIntegerField()

    def __str__(self): return str(self.big_integer)


class PositiveInteger(models.Model):
    positive_integer = models.PositiveIntegerField()

    def __str__(self): return str(self.positive_integer)


class SmallInteger(models.Model):
    small_integer = models.SmallIntegerField()

    def __str__(self): return str(self.small_integer)


class PositiveSmallInteger(models.Model):
    positive_small_integer = models.PositiveSmallIntegerField()

    def __str__(self): return str(self.positive_small_integer)


class OneToOneRelative(models.Model):

    def __str__(self): return str(self.pk)


class OneToOne(models.Model):
    one_to_one = models.OneToOneField(OneToOneRelative, models.CASCADE, related_name='one')

    def __str__(self): return str(self.pk)


class ForeignKey(models.Model):
    foreign_key = models.ForeignKey('app.Integer', on_delete=models.CASCADE)

    def __str__(self): return str(self.pk)


class M2MDependency(models.Model):
    pass


class ManyToMany(models.Model):
    m2m = models.ManyToManyField('app.M2MDependency', blank=True)
    name = models.CharField(max_length=20)

    def __str__(self): return self.name


class GenericIPAddress(models.Model):
    ip_address_v4 = models.GenericIPAddressField(protocol='ipv4')
    ip_address_v6 = models.GenericIPAddressField(protocol='ipv6')
    generic_ip_address = models.GenericIPAddressField()

    def __str__(self): return self.generic_ip_address


class Char(models.Model):
    char = models.CharField(max_length=100)

    def __str__(self): return self.char


class Text(models.Model):
    text = models.TextField()

    def __str__(self): return self.text[:50]


class Uuid(models.Model):
    uuid = models.UUIDField()

    def __str__(self): return str(self.uuid)


class Slug(models.Model):
    slug = models.SlugField()

    def __str__(self): return self.slug


class DateTime(models.Model):
    date_time = models.DateTimeField(null=True, blank=True)
    date_time_auto = models.DateTimeField(auto_now_add=True)

    def __str__(self): return str(self.date_time)


class Date(models.Model):
    date = models.DateField()

    def __str__(self): return str(self.date)


class Time(models.Model):
    time = models.TimeField()

    def __str__(self): return str(self.time)


class Duration(models.Model):
    duration = models.DurationField()

    def __str__(self): return str(self.duration)


class Binary(models.Model):
    binary = models.BinaryField()

    def __str__(self): return str(self.pk)


class AllModel(models.Model):
    boolean = models.BooleanField(default=True)
    char = models.CharField(max_length=1)
    decimal = models.DecimalField(max_digits=5, decimal_places=2)
    file = models.FileField()
    filepath = models.FilePathField(path=settings.BASE_DIR)
    float = models.FloatField(null=True)
    integer = models.IntegerField(null=True)
    big_integer = models.BigIntegerField(null=True)
    generic_ip_address = models.GenericIPAddressField()
    null_boolean = models.NullBooleanField()
    one_to_one = models.OneToOneField('app.Integer', models.CASCADE, related_name='all_model')
    fk = models.OneToOneField('app.Integer', models.CASCADE, related_name='all_models')
    positive_integer = models.PositiveIntegerField()
    positive_small_integer = models.PositiveSmallIntegerField()
    slug = models.SlugField()
    small_integer = models.SmallIntegerField()
    text = models.TextField(null=True)
    time = models.TimeField()
    uuid = models.UUIDField(default=uuid.uuid4)



import os

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-tarantool",
    version="0.0.18",
    package_dir={"django-tarantool": os.path.join("django_tarantool")},
    author="Artem Morozov",
    author_email="artembo@me.com",
    description="Tarantool database backend for Django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/artembo/django-tarantool",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'tarantool>=0.7.1',
    ],
)



from django.db.backends.base.features import BaseDatabaseFeatures
from django.utils.functional import cached_property


class DatabaseFeatures(BaseDatabaseFeatures):
    uses_savepoints = False
    can_alter_table_rename_column = False
    can_introspect_autofield = True  #
    can_introspect_big_integer_field = False
    can_introspect_binary_field = False  #
    can_introspect_decimal_field = False  #
    can_introspect_duration_field = False  #
    can_introspect_ip_address_field = False  #
    can_introspect_time_field = False  #

    max_query_params = 65000

    @cached_property
    def supports_transactions(self):
        return False



from django.core.exceptions import ValidationError
from django.test import TestCase

from app.models import NullBoolean


class NullBooleanTests(TestCase):
    @classmethod
    def setUpClass(cls):
        NullBoolean.objects.create(null_boolean=True)
        NullBoolean.objects.create(null_boolean=False)
        NullBoolean.objects.create(null_boolean=None)

    def test_create_null_boolean(self):
        true = NullBoolean.objects.get(null_boolean=True)
        false = NullBoolean.objects.get(null_boolean=False)
        none = NullBoolean.objects.get(null_boolean=None)
        self.assertTrue(true.null_boolean)
        self.assertFalse(false.null_boolean)
        self.assertIsNone(none.null_boolean)

    def test_wrong_values(self):
        with self.assertRaises(ValidationError):
            NullBoolean.objects.create(null_boolean='string value')
        with self.assertRaises(ValidationError):
            NullBoolean.objects.create(null_boolean=100)



from django.test import TestCase
from django.utils import lorem_ipsum

from app.models import Char


class CharTests(TestCase):
    @classmethod
    def setUpClass(cls):
        char_value = lorem_ipsum.words(5)
        cls.char_value = char_value
        Char.objects.create(char=char_value)
        cls.end = ' '.join(char_value.split()[-2:])

    def test_char_get(self):
        char_item = Char.objects.get(char=self.char_value)
        self.assertEqual(char_item.char, self.char_value)
        with self.assertRaises(Char.DoesNotExist):
            Char.objects.get(char='string which is not in the char')

    def test_char_contains(self):
        self.assertTrue(Char.objects.filter(char__contains='lorem').exists())
        self.assertFalse(Char.objects.filter(char__contains='string which is not in the char').exists())

    def test_char_icontains(self):
        self.assertTrue(Char.objects.filter(char__icontains='lOrEm').exists())
        self.assertFalse(Char.objects.filter(char__icontains='string which is not in the char').exists())

    def test_char_exact(self):
        self.assertTrue(Char.objects.filter(char__exact=self.char_value).exists())
        self.assertFalse(Char.objects.filter(char__exact='string which is not in the char').exists())

    def test_char_iexact(self):
        self.assertTrue(Char.objects.filter(char__iexact=self.char_value.upper()).exists())
        self.assertFalse(Char.objects.filter(char__iexact='string which is not in the char').exists())

    def test_char_startswith(self):
        self.assertTrue(Char.objects.filter(char__startswith='lorem ipsum').exists())
        self.assertFalse(Char.objects.filter(char__startswith='string which is not in the char').exists())

    def test_char_istartswith(self):
        self.assertTrue(Char.objects.filter(char__istartswith='lOreM iPSUm').exists())
        self.assertFalse(Char.objects.filter(char__istartswith='string which is not in the char').exists())

    def test_char_endswith(self):
        self.assertTrue(Char.objects.filter(char__endswith=self.end).exists())
        self.assertFalse(Char.objects.filter(char__endswith='string which is not in the char').exists())

    def test_char_iendswith(self):
        self.assertTrue(Char.objects.filter(char__iendswith=self.end.upper()).exists())
        self.assertFalse(Char.objects.filter(char__iendswith='string which is not in the char').exists())



from django.db import DatabaseError
from django.test import TestCase

from app.models import OneToOneRelative, OneToOne


class OneToOneTests(TestCase):
    @classmethod
    def setUpClass(cls):
        for _ in range(10):
            relative = OneToOneRelative.objects.create()
            OneToOne.objects.create(one_to_one=relative)

    def test_one_to_one(self):
        relative_first = OneToOneRelative.objects.first()
        relative_last = OneToOneRelative.objects.last()
        relative_random = OneToOneRelative.objects.all().order_by('?')[0]
        self.assertEqual(relative_first.id, 1)
        self.assertEqual(relative_last.id, 10)
        self.assertEqual(relative_random.one.one_to_one_id, relative_random.pk)
        self.assertEqual(OneToOne.objects.count(), OneToOneRelative.objects.count())

    def test_values_list(self):
        self.assertListEqual(list(OneToOne.objects.values_list('one_to_one', flat=True)),
                             list(OneToOneRelative.objects.filter(one__isnull=False).values_list('one__id', flat=True)))

    def test_one_to_one_duplicate_relative(self):
        relative = OneToOneRelative.objects.first()
        with self.assertRaises(DatabaseError):
            OneToOne.objects.create(one_to_one=relative)

    def test_one_to_one_invalid(self):
        with self.assertRaises(ValueError):
            OneToOne.objects.create(one_to_one='string value')



from django.test import TestCase
from django.utils import timezone

from app.models import DateTime


class DateTimeTests(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.beginning = timezone.now()
        DateTime.objects.create(date_time=cls.beginning)

    def test_date_time(self):
        date_time = DateTime.objects.get()
        self.assertEqual(date_time.date_time, self.beginning)

        now = timezone.now()
        DateTime.objects.create(date_time=now)
        latest = DateTime.objects.latest('date_time')
        self.assertEqual(now, latest.date_time)
        self.assertGreater(now, date_time.date_time)
